UNIT linlist;

{ -------------------------------------------------------------------
   Begin der Klasse Liste
  -------------------------------------------------------------------}

interface

{ Typen definitionen }
TYPE

{ Definition eines Knotens der einfach verketteten Liste. }
KnotenPtr = ^Knoten;
Knoten = OBJECT
     next: KnotenPtr;
     data: INTEGER;
END;

{ Definition der einfach verketteten Liste. }
ListePtr = ^Liste;
Liste = OBJECT
     start, now: KnotenPtr;
     CONSTRUCTOR Init;
     PROCEDURE addElement (d : integer);
     PROCEDURE clearList;
     PROCEDURE gotoEnd;
     PROCEDURE gotoNext;
     PROCEDURE gotoStart;
     PROCEDURE removeElement;
     PROCEDURE sortList;
     PROCEDURE swapElements (e1, e2 : KnotenPtr);
     FUNCTION eol : boolean;
     FUNCTION find (d: INTEGER) : KnotenPtr;
     FUNCTION getData : INTEGER;
     FUNCTION IsEmpty : boolean;
     FUNCTION NumberOfElements : Integer;
     DESTRUCTOR done;
END;




implementation


{ Constructor Init.
  Setzt alle Anfangswerte. }
constructor Liste.Init;
begin
     start := NIL;
     now := NIL;
end;



{ Procedure addElement ().
  Fgt ein neues Element in der Liste ein.
  @param d Der einzufgende Datenwert. }
procedure Liste.addElement (d : integer);
VAR hptr: KnotenPtr;
begin
     new (hptr);
     hptr^.data := d;

     if find (d) <> NIL then exit;

     { Wenn das erste Element }
     if start <> NIL then
        hptr^.next := start
     else
        hptr^.next := NIL;
     start := hptr;
end;



{ Prozedur clearList.
  Lscht die gesamte Liste. }
procedure Liste.clearList;
begin
     gotoStart;
     while IsEmpty = false do
           removeElement
end;



{ Prozedur gotoEnd.
  Setzt den now Pointer auf das Ende der Liste. }
procedure Liste.gotoEnd;
begin
     while now^.next <> NIL do
           now := now^.next
end;



{ Prozedur gotoNext.
  Setzt den now Pointer auf das nchste Element in der Liste. }
procedure Liste.gotoNext;
begin
     if now = NIL then
        exit;
     if now^.next = NIL then
        exit;
     now := now^.next;
end;



{ Prozedur gotoStart.
  Setzt den now Pointer auf den Anfang der Liste. }
procedure Liste.gotoStart;
begin
     now := start;
end;



{ Prozedur removeElement.
  Lscht das aktuelle Element aus der Liste. }
procedure Liste.removeElement;
var h : KnotenPtr;
begin
     if (start = NIL) or (now = NIL) then
        exit;

     if now <> start then

     { Das nicht-erste Element wird gelscht. }
     begin
          h := start;
          while h^.next <> now do h := h^.next;
          h^.next := now^.next;
          dispose (now);
          now := h;

     { Das erste Element der Liste wird gelscht. }
     end else begin
          start := now^.next;
          dispose (now);
          now := start;
     end
end;



{ Prozedur sortList.
  Sortiert die Liste mit Selection Sort. }
procedure Liste.sortList;
var i, j: integer;
    hstart, kleinstes: KnotenPtr;
begin
     hstart := start;
     for i := 0 to NumberOfElements - 1 do
     begin
          kleinstes := hstart;
          now := hstart^.next;
          while now <> NIL do
          begin
                if now^.data < kleinstes^.data then
                   kleinstes := now;
                now := now^.next;
          end;
          swapElements (kleinstes, hstart);
          hstart := kleinstes^.next;
     end
end;



{ Prozedur swapElements.
  Vertauscht zwei Elemente miteinander.
  @param e1 das erste Element.
  @param e2 das zweite Element. }
procedure Liste.swapElements (e1, e2 : KnotenPtr);
var v1: KnotenPtr;
begin
     v1 := start;

     { Suchen des ersten Vorgngers. }
     if e1 <> start then begin
        while v1^.next <> e1 do v1 := v1^.next;
        v1^.next := e2;
     end else
        start := e2;

     v1 := start;

     { Suchen des zweiten Vorgngers. }
     if e2 <> start then begin
        while v1^.next <> e2 do v1 := v1^.next;
        v1^.next := e1;
     end else
        start := e1;

     { Nachfolger vertauschen. }
     v1 := e1^.next;
     e1^.next := e2^.next;
     e2^.next := v1;
end;



{ Function eol.
  berprft ob der now Pointer auf das letzte Element zeigt.
  @return TRUE, falls das Ende der Liste erreicht wurde. }
function Liste.eol : boolean;
var e: boolean;
begin
     e := FALSE;
     if now = NIL then e := TRUE;
     if now^.next = NIL then
        e := TRUE;
     eol := e;
end;



{ Function find ().
  Sucht nach dem Vorkommen eines Datenelements.
  @param d Zu suchendes Element.
  @return Zeiger auf gefundenes Element.
  }
function Liste.find (d: integer) : KnotenPtr;
var fund: KnotenPtr;
begin
     fund := NIL;
     gotoStart;
     while now <> NIL do
     begin
           if now^.data = d then
           begin
               fund := now;
               now := NIL;
           end else
               now := now^.next;
     end;
     find := fund;
end;



{ Prozedur getData.
  Liefert den Inhalt des aktuellen Elements.
  Keine berprfung ob berhaupt ein Element existiert.
  @return Inhalt des Datenelements. }
function Liste.getData : integer;
begin
     getData := now^.data;
end;



{ Function IsEmpty.
  berprft ob die Liste leer ist.
  @return True = leer; False = nicht leer. }
function Liste.IsEmpty : boolean;
var empty: boolean;
begin
     if start = NIL then
        empty := TRUE
     else
        empty := FALSE;
     IsEmpty := empty;
end;



{ Function NumberOfElements.
  Zhlt die Elemente in der Liste.
  @return Anzahl der Elemente in der Liste. }
function Liste.NumberOfElements : Integer;
var i : integer;
begin
     i := 1;
     gotoStart;
     if now = NIL then
         i := 0
     else
        while now^.next <> NIL do
        begin
           i := i + 1;
           now := now^.next;
        end;
     NumberOfElements := i;
     gotoStart
end;



{ Destructor done.
  Lscht alle Listenelemente wenn Liste nicht leer. }
destructor Liste.done;
begin
     if IsEmpty = false then
        clearList;
end;


{ -------------------------------------------------------------------
   End der Klasse Liste
  -------------------------------------------------------------------}


begin
end.